#!/bin/sh

# xsudo.sh
# allows remote X Window applications to be executed via sudo
# Ben Lincoln, 2010-09-09

if [ $# -lt 2 ]
then
	echo "xsudo.sh - execute remote X applications over an SSH tunnel" >&2
	echo "Usage: `basename $0` TARGETUSER TARGETCOMMAND" >&2
	echo "Note: you may need to specify the -Y flag when calling ssh for this script to work."
	echo "Example: ssh -Y blincoln@tourian"
	exit 2
fi


# change this extension to match your environment
DEFAULTDOMAIN=.domain.local

# figure out the display name that xauth expects

DISPLAYNAME=`echo $DISPLAY | sed -r s/[:/].*//`
DISPLAYNUM=`echo $DISPLAY | sed -r s/[^:]*:// | sed -r s/[\.].*//`

# generate valid permutations as an array
declare -a NAMEARRAY

NAMEARRAY[0]=$DISPLAYNAME:$DISPLAYNUM
NAMEARRAY[1]=$DISPLAYNAME$DEFAULTDOMAIN:$DISPLAYNUM
NAMEARRAY[2]=$DISPLAYNAME/unix:$DISPLAYNUM
NAMEARRAY[1]=$DISPLAYNAME$DEFAULTDOMAIN/unix:$DISPLAYNUM
NAMEARRAY[3]=$HOSTNAME:$DISPLAYNUM
NAMEARRAY[4]=$HOSTNAME$DEFAULTDOMAIN:$DISPLAYNUM
NAMEARRAY[5]=$HOSTNAME/unix:$DISPLAYNUM
NAMEARRAY[4]=$HOSTNAME$DEFAULTDOMAIN/unix:$DISPLAYNUM

ELEMENTS=${#NAMEARRAY[@]}

declare -a XAUTHNAMES
let COUNT=0

for (( i=0; i<$ELEMENTS; i++)); do
	#echo "Checking ${NAMEARRAY[${i}]}"
	#xauth list | grep "${NAMEARRAY[${i}]}"
	CRESULT=`xauth list | grep "${NAMEARRAY[${i}]}"`
	CRL=`expr length "$CRESULT"`
	#expr length "$CRESULT"
	if [ $CRL -gt 1 ]; then
		#echo "Found '$CRESULT'"
		XAUTHNAMES[$COUNT]=$CRESULT
		let COUNT=COUNT+1
	fi
done

ELEMENTS=${#XAUTHNAMES[@]}

for (( i=0; i<$ELEMENTS; i++)); do
	#echo "xauth add ${XAUTHNAMES[${i}]}"
	sudo -H -u $1 xauth add ${XAUTHNAMES[${i}]}
done

sudo -H -u $1 $2

# clean up afterwards
for (( i=0; i<$ELEMENTS; i++)); do
	#echo "xauth remove ${XAUTHNAMES[${i}]}"
	DISPLAYNAME=`echo ${XAUTHNAMES[${i}]} | sed -r "s/[ ].*//"`
	sudo -H -u $1 xauth remove $DISPLAYNAME
done
